﻿--[[ Guild Management Tools
  -- URL: http://addons.nobilityguild.org/
  -- 
  -- License:
  -- 	This program is free software; you can redistribute it and/or
  -- 	modify it under the terms of the GNU General Public License
  -- 	as published by the Free Software Foundation; either version 2
  -- 	of the License, or (at your option) any later version.
  -- 	This program is distributed in the hope that it will be useful,
  -- 	but WITHOUT ANY WARRANTY; without even the implied warranty of
  -- 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  -- 	GNU General Public License for more details.
  -- 	You should have received a copy of the GNU General Public License
  -- 	along with this program(see GPL.txt); if not, write to the Free Software
  -- 	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
--]]
--
--[[ 1.0     Addon Declarations
  -- 1.1         Localizations using AceLocale-3.0
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("GuildManagementTools", false)


--~ 1.2         Registering Addons
--[[ 1.2.1           LibDebugLog -   Used to control the diaply of various debug messages; this library is missing some desired features. I will probably write this into the LibGMT library at some point.
  -- 1.2.1.1             Mixin API
  -- 1.2.1.1.1               :Debug( msg [, ...] )
  --                             Sends a debug log message.
  --                             msg is converted to string. If more arguments are provided, they are converted to strings. If msg contains at least one '%', sends msg:format(...), else join all arguments.
  -- 1.2.1.1.2               :ToggleDebugLog( enable )
  --                             Enable debug logging if enable is true.
  -- 1.2.1.1.3               :IsDebugLogEnabled()
  --                             Returns true if debug logs are enabled.
  -- 1.2.1.2             Library API
  --                         These methods have to be called from the library itself, e.g. :
  --                             LibStub('LibLogDebug-1.0'):SetGlobalToggle(false)
  -- 1.2.1.2.1               :Embed(broker)
  --                             Add the embeddable API into the broker and registers the broker.
  -- 1.2.1.2.2               :SetGlobalToggle( enable )
  --                             Forcibly set the log state of all registered brokers.
  --                                 Possible values of enable are:
  --                                     true    -   forcibly enable debug log.
  --                                     false   -   forcibly disable debug log.
  --                                     nil     -   restores each broker state.
  -- 1.2.1.2.3               :GetGlobalToggle()
  --                             Returns the current state of the global toggle.
  -- 1.2.1.2.4               :GetAce3OptionTable( logger[, order] )
  --                             Returns a table to be used as an AceConfig3 option table.
  --                                 function addon:OnInitialize()
  --                                   local options = {
  --                                     name = 'MyAddon',
  --                                     type = 'group',
  --                                     handler = self,
  --                                     args = {
  --                                       enable = {
  --                                         name = 'Enable',
  --                                         type = 'toggle',
  --                                         get = 'IsEnabled',
  --                                         set = 'EnableAddon',
  --                                         order = 100,
  --                                       },
  --                                       debug = LibStub('LibLogDebug-1.0'):GetAce3OptionTable(self, 110),
  --                                       -- put other useful options here
  --                                     }
  --                                   }
  --                                   LibStub("AceConfig-3.0"):RegisterOptionsTable(self.name, options)
  --                                 end
--]]

--[[ 1.2.2           AceEvent
  -- 1.2.2.1             Subscribing to events
  -- 1.2.2.1.1               MyAddon:RegisterEvent("NAME_OF_EVENT", "MyHandlerMethod")
  --                             function MyAddon:NAME_OF_EVENT(eventName, arg1, arg2, arg3)
  --                             -- do some stuff
  --                             end
  -- 
  --                             function MyAddon:NAME_OF_OTHER_EVENT(eventName, ...)
  --                             -- do some more stuff
  --                             end
  -- 1.2.2.1.2               MyAddon:UnregisterEvent("NAME_OF_EVENT")
  -- 1.2.2.2             Sending/receiving inter-addon messages
  --                         AceEvent also provides support for "messages", which are basically like events except instead of being triggered by the WoW client, they're triggered by other addons. This is useful if you have multiple addons which need to talk to each other. Messages are subscribed to very similarly to events (and the alternate options for specifying handlers work for RegisterMessage, too):
  -- 1.2.2.2.1               MyAddon:RegisterMessage("NAME_OF_MESSAGE")
  -- 
  --                             function MyAddon:NAME_OF_MESSAGE()
  --                                 -- handle the message
  --                             end
  -- 1.2.2.2.2               MyAddon:UnregisterMessage("NAME_OF_MESSAGE")
  -- 1.2.2.2.3               Sending messages to other addons is just as simple:
  --                             MyAddon:SendMessage("NAME_OF_MESSAGE")
  --                             MyAddon:SendMessage("NAME_OF_OTHER_MESSAGE", arg1, arg2)
  --                             Note that messages are only for addons running on the same client! If you want to send messages between various players and their addons, you'll want to use AceComm.
  -- --
--]]
--[[ 1.2.3           AceTimer
  --                     This creates a timer, to call the function self:UpdatePartyGUIDs() after 0.5 seconds, but only if a timer to do it doesn't already exist.
  --                     Code:
  --                         if not timers.UpdatePartyGUIDs then
  --                             timers.UpdatePartyGUIDs = self:ScheduleTimer("UpdatePartyGUIDs", 0.5)
  --                         end
  --                     Now this is a one-shot timer, its not a repeating timer. When it fires, the handle no longer becomes valid, you need to nil out the invalid timer to it.
  --                     Code:
  --                     function Omen:UpdatePartyGUIDs()
  --                         timers.UpdatePartyGUIDs = nil
  --                         -- blah blah rest of code
  --                     end
  --                     Now, the function UpdatePartyGUIDs() can be called from many places, not just from a timer. So we cancel the timer if it exists (that is we cancel the timer if some other code calls it ahead of the timer), and also have it not error on a invalid timer handle by sending it "true" in the second argument.
  --                     Code:
  --                     function Omen:UpdatePartyGUIDs()
  --                         if timers.UpdatePartyGUIDs then
  --                             self:CancelTimer(timers.UpdatePartyGUIDs, true)
  --                             timers.UpdatePartyGUIDs = nil
  --                         end
  --                         -- blah blah rest of code
  --                     end
  -- --
--]]
--[[ 1.2.4           AceBucket
  --                     Not Currently Used - Marked for deletion
--]]





GuildManagementTools = LibStub("AceAddon-3.0"):NewAddon("Guild Management Tools", "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0")
local MODNAME = "Guild Management Tools"
local addon = LibStub("AceAddon-3.0"):GetAddon(MODNAME)

--~ Section Break
--~ 1.3         Localize global functions and variables
local BOOKTYPE_SPELL = BOOKTYPE_SPELL
local _G = _G
local tconcat = table.concat
local tinsert = table.insert
local tmaxn = table.maxn
local tremove = table.remove
local tsort = table.sort
local math_abs = math.abs
local math_ceil = math.ceil
local math_deg = math.deg
local math_exp = math.exp
local math_floor = math.floor
local math_fmod = math.fmod
local math_log = math.log
local math_log10 = math.log10
local math_max = math.max
local math_min = math.min
local math_modf = math.modf
local math_pi = math.pi
local math_pow = math.pow
local math_rad = math.rad
local math_random = math.random
local math_randomseed = math.randomseed
local math_sqrt = math.sqrt
local string_format = string.format
local string_len = string.len
local string_lower = string.lower
local string_rep = string.rep
local string_reverse = string.reverse
local string_sub = string.sub
local string_upper = string.upper
local string_find = string.find
local pairs = pairs
local select = select
local type = type
local tostring = tostring
-- colors
local RED		= "ff0000"
local ORANGE	= "ff7f00"
local YELLOW	= "ffff00"
local GREEN		= "00ff00"
local WHITE		= "ffffff"
local COPPER	= "eda55f"
local SILVER	= "c7c7cf"
local GOLD		= "ffd700"
local PURPLE	= "9980CC"
local BLUE		= "0000ff"
local CYAN		= "00ffff"
local BLACK		= "000000"
local GREY		= "666666"
local MIDGREY	= "858585"
local NEUTRAL	= "bfbfbf"
local FRIENDLY	= WHITE
local HONORED	= "00ff00"
local REVERED	= "3f66e5"
local EXALTED	= "9933cc"
local POOR		= "919191"
local COMMON	= WHITE
local UNCOMMON	= "00A900"
local RARE		= "0062C3"
local EPIC		= "B343FF"
local LEGENDARY	= "FA9900"
local ARTIFACT	= "e5cc80"
local HIGH		= WHITE
local NORMAL	= "ffd100"
local HORDE		= RED
local ALLIANCE	= "00ffff"

local sec_hour = 3600
local sec_day = 86400
local sec_year = 31536000
-- 30_day_Month = 2592000
-- 31_day_month = 2678400
-- 29_day_month = 2505600

local month_sec = {
   [1] = 2678400,	--31
   [2] = 2505600,	--29
   [3] = 2678400,	--31
   [4] = 2592000,	--30
   [5] = 2678400,	--31
   [6] = 2592000,	--30
   [7] = 2678400,	--31
   [8] = 2678400,	--31
   [9] = 2592000,	--30
   [10] = 2678400,	--31
   [11] = 2592000,	--30
   [12] = 2678400,	--31
}
--  DEBUG ENGINE
-- Message Levels
DEBUG_CRITICAL = 1 -- used for critical errors which might crash the addon or
               -- result in corrupted data (especially in the db)
DEBUG_ERROR    = 2 -- used for non-critical errors which won't end up in any addon
               -- non-responding or result in incorrect data
DEBUG_WARNING  = 3 -- used for states which might cause an error, if they are 
               -- unexpected
DEBUG_NOTICE   = 4 -- used to give notice of current states which might be useful
               -- and clarify the current functional behaviour
DEBUG_INFO     = 5 -- used to inform of less important states which still might be
               -- useful to track down how the code behaves
DEBUG_DEBUG    = 6 -- used for local debugging, only

debugLog = {
	messages = {},
	levels = {
		[DEBUG_CRITICAL] = "Critical",
		[DEBUG_ERROR]    = "Error",
		[DEBUG_WARNING]  = "Warning",
		[DEBUG_NOTICE]   = "Notice",
		[DEBUG_INFO]     = "Info",
		[DEBUG_DEBUG]    = "Debug",
	}
}
--


if (not GMTdb) then GMTdb ={} end
if (not GMTAceDB) then GMTAceDB = {} end
if (not GMTDebugLog) then GMTDebugLog = {} end
local private ={}

do 
    local gmtTimers ={}
    local gmtDefaults = {
            profile = {
                gmtDebugFlag = nil,
                gmtUpdatePlayerInfoInterval = 20,
                gmtUpdatePlayerGuildInfoInterval = 60,
            },
        }
end

function GuildManagementTools:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("GMTAceDB")
    
    local gmtOptions = {
        name = "|CFF6633FFGuild Management Tools",
        type = "group",
        order = 0,
            args = {
                header = {
                    name = "|CFF6633FFBasic Guild Management Tools Settings",
                    type = "description",
                    order = 0,
                },
                line2 = {
                    name = "Base Module Settings",
                    type = "description",
                    order = 1,
                },
            },
    }
   
   print(L["GMT_VER"]); 
   print(L["USE_SLASH"])
end
--
function GuildManagementTools:OnEnable()
    GuildManagementTools:RegisterEvent("ACHIEVEMENT_EARNED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("ADDON_LOADED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("CHAT_MSG_ADDON", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("HONOR_CURRENCY_UPDATE", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("KNOWN_TITLES_UPDATE", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("LEARNED_SPELL_IN_TAB", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("NEW_TITLE_EARNED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_ENTERING_WORLD", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_GUILD_UPDATE", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_LEVEL_UP", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_LOGIN", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_LOGOUT", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_TARGET_CHANGED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("PLAYER_XP_UPDATE", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("SKILL_LINES_CHANGED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("SPELLS_CHANGED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("TIME_PLAYED_MSG", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("TRADE_SKILL_CLOSE", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("TRADE_SKILL_SHOW", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("TRADE_SKILL_UPDATE", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("TRAINER_CLOSED", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("TRAINER_SHOW", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("UPDATE_FACTION", "DefaultEventHandler")
    GuildManagementTools:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "DefaultEventHandler")
    
end
--
function GuildManagementTools:OnDisable()
end
--
function GuildManagementTools:DefaultEventHandler()

end
--
local function argsToString(arg, ...)
    if select("#", ...) > 0 then
        return tostring(arg), argsToString(...)
    else
        return tostring(arg)
    end
end
--
function print(...)
    local output = strjoin(", ", argsToString(...))
    local text = "|CFF3399FFGuildManagementTools".."|r: "
    local text = text..output
    ChatFrame1:AddMessage(text)
end
--
function RGBtoHEX(r,g,b)
	return string_format("%2x%2x%2x", r*255, g*255, b*255)
end
--
function Colourize(hexColour, text)

	return "|cff" .. tostring(hexColour or 'ffffff') .. tostring(text) .. "|r"

end
--
local function GetRealEventTime(dYear, dMonth, dDay, dHour)
   dYear = dYear or 0
   dMonth = dMonth or 0
   dDay = dDay or 0
   dHour = dHour or 0
   local dif
   dif = dHour*sec_hour + (dDay*sec_day) + GetSecFromMonth(dMonth, date("%m")) + (dYear*sec_year)
   return string.lower(date("%m/%d/%Y %I%p", (time() - GetOffSetTime() - dif)))
end
--
local function DebugLog(mAddon, mType, mLevel, mTitle, ...)
    if broken then return end
    if not (GMTAceDB.gmtDebugFlag and mAddon and mType and mLevel and mTitle) then return end
    local ts = date("%b %d %H:%M:%S");
	pid = pid + 1
	local msg = format(...)
    
    
end
--